#define F_CPU 16000000

#include <avr\io.h>
#include <util/delay.h>
#include <stdlib.h>

int mat[8][8], gameEnded, length;

int isUpPressed(){
	if((PIND & (1<<PD2))){
		return 1;
	}
	return 0;
}

int isRightPressed(){
	if(!(PIND & (1<<PD3))){
		return 1;
	}
	return 0;
}

int isDownPressed(){
	if(!(PIND & (1<<PD4))){
		return 1;
	}
	return 0;
}

int isLeftPressed(){
	if(!(PIND & (1<<PD5))){
		return 1;
	}
	return 0;
}

int isSpecialPressed(){
	if(!(PIND & (1<<PD6))){
		return 1;
	}
	return 0;
}

void generateDot(){
	int dotX, dotY;
	do{
		dotX = rand() % 8;
		dotY = rand() % 8;
	} while(mat[dotY][dotX] >= 0);
	
	mat[dotY][dotX] = 5;
}

void game_ended(){
	int i, j;
	
	//game was won	
	//print OK
	if(length >= 20){
		for(i=0; i<8; i++)	{
			for(j=0; j<8; j++)	{
				mat[i][j] = -1;
			}
		}
		for(i=1; i<7; i++)	{
			mat[i][0] = 0;
			mat[i][2] = 0;
			mat[i][4] = 0;
		}
		mat[1][1] = mat[6][1] = 0;
		
		mat[1][7] = mat[6][7] = 0;
		mat[2][6] = mat[5][6] = 0;
		mat[3][5] = mat[4][5] = 0;		
	}
	else{
		for(i=0; i<8; i++)	{
			for(j=0; j<8; j++)	{
				if(mat[i][j] == 5)
					mat[i][j] = -1;
			}
		}
	}
	
	length = 3;
	gameEnded = 1;
}

void printMatrix(){
	int i, j, k;
	
	
	for(k=0; k<56 - (length * 2); k++) {
		
		PORTB = 0x00;
		PORTA = 0xFF;
		
		for(i=0; i<8; i++)	{
			switch(i) {
				case 0:
					PORTB |= (1 << PB6);
					break;
				case 1:
					PORTB |= (1 << PB7);
					break;
				case 2:
					PORTB |= (1 << PB0);
					break;
				case 3:
					PORTB |= (1 << PB1);
					break;
				case 4:
					PORTB |= (1 << PB5);
					break;
				case 5:
					PORTB |= (1 << PB4);
					break;
				case 6:
					PORTB |= (1 << PB2);
					break;
				case 7:
					PORTB |= (1 << PB3);
					break;
			}
			for(j=0; j<8; j++)	{
				if(mat[i][j] >= 0){
					switch(j) {
						case 0:
							PORTA &= ~(1 << PA0);
							break;
						case 1:
							PORTA &= ~(1 << PA6);
							break;
						case 2:
							PORTA &= ~(1 << PA1);
							break;
						case 3:
							PORTA &= ~(1 << PA4);
							break;
						case 4:
							PORTA &= ~(1 << PA3);
							break;
						case 5:
							PORTA &= ~(1 << PA7);
							break;
						case 6:
							PORTA &= ~(1 << PA5);
							break;
						case 7:
							PORTA &= ~(1 << PA2);
							break;
					}
				}
			}
			_delay_ms(1);
			PORTA = 0xFF;
			PORTB = 0x00;
		}
	}
}

int main()
{
	DDRA = 0xFF;
	DDRB = 0xFF;
  
	DDRD |= (1 << PD7);
	PORTD = 0xFF;
	
	PORTD &= ~(1<< PD7);
	PORTD &= ~(1<< PD2);
	
	int firstX, firstY, lastX, lastY, direction;
	//0 - up
	//1 - right
	//2 - down
	//3 - left
	int i, j, k;
	int mode = 0; 
	length = 3; 
	gameEnded = 0;
	
	for(i=0; i<8; i++)	{
		for(j=0; j<8; j++)	{
			mat[i][j] = -1;
		}
	}
	mat[3][0] = 1;
	mat[3][1] = 1;
	mat[3][2] = 4;
	firstY = 3;
	firstX = 2;
	lastY = 3;
	lastX = 0;
	direction = -1;
	
	_delay_ms(1000);
	while (1){
	
		if(direction == -1){
			//CHANGE MODE
			if(isSpecialPressed()){
				if(mode == 0){
					PORTD |= (1<<PD7);
					mode = 1;
				}
				else if(mode == 1){
					PORTD &= ~(1<<PD7);
					mode = 0;
				}
				_delay_ms(1000);
			}
			else if(isUpPressed()){
				direction = 0;
				generateDot();
			}
			else if(isRightPressed()){
				direction = 1;
				generateDot();
			}
			else if(isDownPressed()){
				direction = 2;
				generateDot();
			}
		}
		else {
			//RESTART
			if(isSpecialPressed()){
				for(i=0; i<8; i++)	{
					for(j=0; j<8; j++)	{
						mat[i][j] = -1;
					}
				}
				mat[3][0] = 1;
				mat[3][1] = 1;
				mat[3][2] = 4;
				firstY = 3;
				firstX = 2;
				lastY = 3;
				lastX = 0;
				direction = -1;
				gameEnded = 0;
				_delay_ms(1000);
				continue;
			}
			if(gameEnded == 1){
				_delay_ms(500);
			}
			else {
				if(isUpPressed() && direction != 2){
					direction = 0;
				}
				else if(isRightPressed() && direction != 3){
					direction = 1;
				}
				else if(isDownPressed() && direction != 0){
					direction = 2;
				}
				else if(isLeftPressed() && direction != 1){
					direction = 3;
				}
				
				switch(direction) {
					case 0:
						mat[firstY][firstX] = 0;
						firstY--;
						if(firstY == -1){
							if(mode == 0)
								firstY = 7;
							else{
								firstY++;
								game_ended();
								continue;
							}
						}
						break;
					case 1:
						mat[firstY][firstX] = 1;
						firstX++;
						if(firstX == 8){
							if(mode == 0)
								firstX = 0;
							else{
								firstX--;
								game_ended();
								continue;
							}
						}
						break;
					case 2:
						mat[firstY][firstX] = 2;
						firstY++;
						if(firstY == 8){
							if(mode == 0)
								firstY = 0;
							else{
								firstY--;
								game_ended();
								continue;
							}
						}
						break;
					case 3:
						mat[firstY][firstX] = 3;
						firstX--;
						if(firstX == -1){
							if(mode == 0)
								firstX = 7;
							else{
								firstX++;
								game_ended();
								continue;
							}
						}
						break;
				}
				if(mat[firstY][firstX] >= 0 && mat[firstY][firstX] < 5){
					game_ended();
					continue;
				}
							
				if(mat[firstY][firstX] == 5){
					generateDot();
					length++;
				}
				else{				
					switch(mat[lastY][lastX]) {
						case 0:
							mat[lastY][lastX] = -1;
							lastY = (lastY == 0) ? 7 : (lastY - 1);
							break;
						case 1:
							mat[lastY][lastX] = -1;
							lastX = (lastX + 1) % 8;
							break;
						case 2:
							mat[lastY][lastX] = -1;
							lastY = (lastY + 1) % 8;
							break;
						case 3:
							mat[lastY][lastX] = -1;
							lastX = (lastX == 0) ? 7 : (lastX - 1);
							break;
					}
				}
				mat[firstY][firstX] = 4;
				
			}		
		}
		
		printMatrix();
		
	}
	
	return 0;
}

